#-------------------------------------------------------------------------
# notes:
# all "data" or "peaks" entries have the following data items defined 
# (for entry "dat"):
#    dat(type) datatype == xy (data) or peaks
#    dat(xvector) name of x vector (dat_x)
#    dat(yvector) name of y vector (dat_y)
#    dat(x) x data -- as read
#    dat(y) intensity data (or peak areas)
#    dat(line) line type be plotted
#    dat(color) color to be plotted
#    dat(symbol) symbol type  be plotted
#    dat(symsize) relative symbol size be plotted
#    dat(title) descriptive text
#    dat(xlabel) x axis label for raw data
#    dat(ylabel) y axis label for raw data
#    dat(cxlabel) x axis label for data in current units
#    dat(cylabel) y axis label for data in current units
#
#  data "file" dat has the following array elements defined:
#    dat(esdvec) name of esd vector (dat_esd)
#    dat(y)  y data -- as read
#    dat(esd)  esd data -- as read
#  data "file" dat has the following vectors defined:
#     dat_x     x axis
#     dat_y     y axis
#     dat_esd   esd on y axis
#        these are derived from dat(x), dat(y) and dat(z)
# 
# files of peaks are handled a bit differently
#   peak "file" dat has the following array elements defined:
#     dat(h) dat(k) dat(l) hkl for reflections
#     dat(ttvector) name of two-theta vector, dat_tt
    # _tt is used for reflection labeling (lblhkl)
#     dat(extinctions) {} 1 for extinct peaks
#     dat(dspaces) {}
#     dat(wavelength) ""
#     dat(a) dat(b) dat(c) dat(alpha) dat(beta) dat(gamma) 
#     dat(old_a) dat(old_b) dat(old_c) 
#     dat(old_alpha) dat(old_beta) dat(old_gamma) 
#     dat(heights) peak heights 
#     dat(widths) intensity data (or peak areas)
#     dat(bkgs)  background values
#  peak "file" dat has the following vectors defined:
#     dat_x     x axis
#     dat_y     y axis
#     dat_tt
# note that the name "dat" must be added to list graph(datalist) to be used
#
#########################################################################
proc initdata {data} {
    global colorlist linelist ncolors nlines colorindex lineindex
    global graph
    global $data 
    regsub -all  {\.} $data {} cdat
    regsub -all  {[\*\+-]} $cdat {_} cdat
    # define vectors for the data
    set ${data}(type) xy
    set ${data}(xvector) ${cdat}_x
    set ${data}(yvector) ${cdat}_y
    set ${data}(esdvec) ${cdat}_esd
    global [set ${data}(xvector)] [set ${data}(yvector)] [set ${data}(esdvec)]
    catch {vector [set ${data}(xvector)]}
    catch {vector [set ${data}(yvector)]}
    catch {vector [set ${data}(esdvec)]}
    set ${data}(wavelength) ""
    set ${data}(xunits) ""
    set ${data}(xlabel) ""
    set ${data}(ylabel) ""
    set ${data}(cxlabel) ""
    set ${data}(cylabel) ""
    set ${data}(title) ""
    # initialize arrays
    set ${data}(x) ""
    set ${data}(y) ""
    set ${data}(esd) ""
    # set the color and line type
    incr colorindex 
    if {$colorindex == $ncolors} {set colorindex 0}
    incr lineindex 
    if {$lineindex == $nlines} {set lineindex 0}
    set ${data}(line) 1
    set ${data}(color) [lindex $colorlist $colorindex]
    set ${data}(symbol) [lindex $linelist $lineindex]
    set ${data}(symsize) 1.0
    # add to list of defined arrays
    if {[lsearch -exact $graph(datalist) $data] == -1} {
	lappend graph(datalist) $data
    }
}

proc initpeaks {data} {
    global colorlist linelist ncolors nlines colorindex lineindex
    global graph
    global $data 
    # define vectors for the data
    set ${data}(type) peaks
    regsub -all  {\.} $data {} cdat
    regsub -all  {[\*\+-]} $cdat {_} cdat
    set ${data}(xvector) ${cdat}_x
    set ${data}(yvector) ${cdat}_y
    set ${data}(ttvector) ${cdat}_tt
    global [set ${data}(xvector)] [set ${data}(yvector)] [set ${data}(ttvector)]
    catch {vector [set ${data}(xvector)]}
    catch {vector [set ${data}(yvector)]}
    catch {vector [set ${data}(ttvector)]}
    # initialize arrays
    set ${data}(x) ""
    set ${data}(y) ""
    set ${data}(h) {}
    set ${data}(k) {}
    set ${data}(l) {}
    set ${data}(extinctions) {}
    set ${data}(dspaces) {}
    set ${data}(widths) {}
    set ${data}(bkgs) {}
    set ${data}(heights) {}
    set ${data}(wavelength) ""
    set ${data}(xunits) ""
    set ${data}(xlabel) ""
    set ${data}(ylabel) ""
    set ${data}(cxlabel) ""
    set ${data}(cylabel) ""
    set ${data}(title) ""
    set ${data}(spg) ""
    set ${data}(extcodes) ""
    # set the color and line type
    set ${data}(line) 1
    incr colorindex 
    if {$colorindex == $ncolors} {set colorindex 0}
    incr lineindex 
    if {$lineindex == $nlines} {set lineindex 0}
    set ${data}(color) [lindex $colorlist $colorindex]
    set ${data}(symbol) [lindex $linelist $lineindex]
    set ${data}(symsize) 1.0
    # add to list of defined arrays
    if {[lsearch -exact $graph(datalist) $data] == -1} {
	lappend graph(datalist) $data
    }
    foreach var {a b c alpha beta gamma} {
	set ${data}(old_$var) {}
	set ${data}($var) {}
    }
}

proc peaktovector {data} {
    global $data cellparm
    set xtmp {}
    set ytmp {}
    # put the peaks in order
    set l {}
    foreach peak [set ${data}(x)] \
	    h [set ${data}(heights)] \
	    b [set ${data}(bkgs)] {
	if ![catch {expr $peak}] {
	    lappend l [list $peak $h $b]
	}
    }
    foreach triple [lsort -index 0 -real $l] {
	set peak [lindex $triple 0]
	set h [lindex $triple 1]
	set b [lindex $triple 2]
	if {$peak > 0} {
	    lappend xtmp $peak $peak $peak
	    set ymin $cellparm(ymin)
	    set ymax $cellparm(ymax)
	    catch {
		if {$h != "" &&  $h > 0} {
		    set ymin $b
		    set ymax [expr $h+$b]
		}
	    }
	    lappend ytmp $ymin $ymax $ymin
	}
    }
    [set ${data}(ttvector)] set [set ${data}(x)]
    [set ${data}(xvector)] notify never
    [set ${data}(yvector)] notify never
    [set ${data}(xvector)] set $xtmp
    [set ${data}(yvector)] set $ytmp
    [set ${data}(xvector)] notify now
    [set ${data}(yvector)] notify now
    [set ${data}(xvector)] notify always
    [set ${data}(yvector)] notify always
}

proc resetdata {data} {
    global $data 
    if {[set ${data}(type)] == "xy"} {
	[set ${data}(xvector)] set [set ${data}(x)]
	# make sure that all data arrays are the same length as the data
	foreach elem {y esd} {
	    for  {set i [llength [set ${data}($elem)]]} \
		    {$i < [llength [set ${data}(x)]]} {incr i} {
		lappend ${data}($elem) -1
	    }
	}
	[set ${data}(yvector)] set [set ${data}(y)]
	[set ${data}(esdvec)] set [set ${data}(esd)]
    } elseif {[set ${data}(type)] == "peaks"} {
	# make sure that all peak arrays are the same length as the data
	foreach elem {y widths heights bkgs extinctions h k l dspaces} {
	    for  {set i [llength [set ${data}($elem)]]} \
		    {$i < [llength [set ${data}(x)]]} {incr i} {
		lappend ${data}($elem) {}
	    }
	}
	peaktovector $data
    }
    set ${data}(cxlabel) [set ${data}(xlabel)]
    set ${data}(cylabel) [set ${data}(ylabel)]
}

proc scaledata {data xoffset xmult yoffset ymult} {
    # scale x values as xoffset + (xmult * data)
    global $data
    if {$xmult != 1} {[set ${data}(xvector)] set [[set ${data}(xvector)] * $xmult]}
    if {$xoffset != 0} {[set ${data}(xvector)] set [[set ${data}(xvector)] + $xoffset]}
    if {$ymult != 1} {
	[set ${data}(yvector)] set [[set ${data}(yvector)] * $ymult]
	if {[set ${data}(type)] == "xy"} {
	    [set ${data}(esdvec)] set [[set ${data}(esdvec)] * $ymult]
	}
    }
    if {$yoffset != 0} {[set ${data}(yvector)] set [[set ${data}(yvector)] + $yoffset]}
}

proc ConvertY {data mode} {
    # convert intensities to sqrt(I), log(I), I/sig(I)
    global $data
    if {$mode == ""} {
	return
    }
    if {$mode == "sqrt"} {
	foreach y [[set ${data}(yvector)] range 0 end] {
	    set yt 0.0
	    catch {set yt [expr sqrt($y)]}
	    lappend ytmp $yt
	}
	[set ${data}(yvector)] set $ytmp
	set ${data}(cylabel) "sqrt(I)"
    } elseif {$mode == "log"} {
	foreach y [[set ${data}(yvector)] range 0 end] {
	    set yt 0.0
	    catch {set yt [expr log10($y)]}
	    lappend ytmp $yt
	}
	set ${data}(cylabel) "log(I)"
	[set ${data}(yvector)] set $ytmp
    } elseif {$mode == "s-n"} {
	foreach y   [[set ${data}(yvector)] range 0 end] \
		err [[set ${data}(esdvec)]  range 0 end] {
	    set yt 0.0
	    catch {set yt [expr $y/$err]}
	    lappend ytmp $yt
	}
	set ${data}(cylabel) "I/sigma_I"
	[set ${data}(yvector)] set $ytmp
    } elseif {$mode == "unscale"} {
	foreach y   [[set ${data}(yvector)] range 0 end] \
		err [[set ${data}(esdvec)]  range 0 end] {
	    set yt 0.0
	    catch {set yt [expr {($y/$err)*($y/$err)}]}
	    lappend ytmp $yt
	    set et 0.0
	    catch {set et [expr {($y/$err)}]}
	    lappend etmp $et
	}
	set ${data}(cylabel) "unscaled counts"
	[set ${data}(yvector)] set $ytmp
	[set ${data}(esdvec)] set $etmp
    }
}

proc ConvertX {data mode {wave ""}} {
    # convert x-axis to 2theta, Q or dspace 
    global command
    global $data
    if {$mode == ""} {
	return
    }
    if {$mode == "2theta"} {
	# check the conversion wavelength
	if [catch {expr $wave}] {
	    tk_dialog .warn "Invalid wavelength" \
	"Unable to convert data to 2-theta units: \
	Invalid conversion wavelength, \"$wave\"" warning 0 OK
	return
	}
    }
    if {[set ${data}(xunits)] == "2theta"} {
	while {[catch "expr [set ${data}(wavelength)]"]} {
	    catch {destroy .wave}
	    toplevel .wave
	    wm title .wave "Get wavelength"
	    pack [label .wave.0 -text \
		    "Enter a wavelength for dataset $data"]
	    pack [frame .wave.1] -side top -expand yes -fill both
	    pack [label .wave.1.a -text Wavelength] -side left 
	    pack [entry .wave.1.b -textvariable ${data}(wavelength)] -side left 
	    pack [button .wave.2 -text OK -command "destroy .wave"] -side top 
	    # reuse the last value as the next default
	    catch {set [set data](wavelength) $command(lastwavelength)}	    
	    grab .wave
	    tkwait window .wave
	    set command(lastwavelength) [set ${data}(wavelength)]
	}
	if {$mode == "2theta"} {
	    set torad [expr 3.14159 / 360.]
	    set wrat [expr $wave / [set ${data}(wavelength)]]
	    foreach x [[set ${data}(xvector)] range 0 end] {
		set xt 180.0
		catch {set xt [expr asin($wrat * sin($x*$torad))/$torad ]}
		lappend xtmp $xt
	    }
	    [set ${data}(xvector)] set $xtmp
	    set ${data}(cxlabel) "2 Theta @ $wave"
	} elseif {$mode == "Q"} {
	    set torad [expr 3.14159 / 360.]
	    set pi4ow [expr  4. * 3.14159 / \
		    [set ${data}(wavelength)]]
	    foreach x [[set ${data}(xvector)] range 0 end] {
		set xt 0.0
		catch {set xt [expr $pi4ow * sin($x*$torad)]}
		lappend xtmp $xt
	    }
	    [set ${data}(xvector)] set $xtmp
	    set ${data}(cxlabel) "Q"
	} elseif {$mode == "dspace"} {
	    set torad [expr 3.14159 / 360.]
	    set wo2 [expr 0.5 * [set ${data}(wavelength)]]
	    foreach x [[set ${data}(xvector)] range 0 end] {
		set xt 0.0
		catch {set xt [expr $wo2 / sin($x*$torad) ]} err
		lappend xtmp $xt
	    }
	    set ${data}(cxlabel) "d-space"
	    [set ${data}(xvector)] set $xtmp
	}
	return
    } elseif {[set ${data}(xunits)] == "dspace"} {
	if {$mode == "2theta"} {
	    set torad [expr 3.14159 / 360.]
	    set wo2 [expr $wave / 2.]
	    foreach x [[set ${data}(xvector)] range 0 end] {
		set xt 180.0
		catch {set xt [expr asin( $wo2/$x )/$torad ]}
		lappend xtmp $xt
	    }
	    [set ${data}(xvector)] set $xtmp
	    set ${data}(cxlabel) "2 Theta @ $wave"
	} elseif {$mode == "Q"} {
	    set pi4 [expr  4. * 3.14159 ]
	    foreach x [[set ${data}(xvector)] range 0 end] {
		set xt 0.0
		catch {set xt [expr $pi4 / $x]}
		lappend xtmp $xt
	    }
	    [set ${data}(xvector)] set $xtmp
	    set ${data}(cxlabel) "Q"
	} elseif {$mode == "dspace"} {
	    return
	}
    } elseif {[set ${data}(xunits)] == "EDSKEV"} {
	while {[catch "expr [set ${data}(wavelength)]"]} {
	    catch {destroy .wave}
	    toplevel .wave
	    wm title .wave "Get 2Theta"
	    pack [label .wave.0 -text \
		    "Enter the 2Theta value for dataset $data"]
	    pack [frame .wave.1] -side top -expand yes -fill both
	    pack [label .wave.1.a -text 2Theta] -side left 
	    pack [entry .wave.1.b -textvariable ${data}(wavelength)] -side left 
	    pack [button .wave.2 -text OK -command "destroy .wave"] -side top 
	    # reuse the last value as the next default
	    catch {set [set data](wavelength) $command(lastwavelength)}	    
	    grab .wave
	    tkwait window .wave
	    set command(lastwavelength) [set ${data}(wavelength)]
	}
	if {$mode == "2theta"} {
	    set torad [expr 3.14159 / 360.]
	    set wo2 [expr $wave / 2.]
	    # wavelength is 2theta for EDS
	    set cnv [expr 6.199 / sin ($torad * [set ${data}(wavelength)])]
	    foreach x [[set ${data}(xvector)] range 0 end] {
		set xt 180.0
		catch {set xt [expr asin( $wo2*$x/$cnv )/$torad ]}
		lappend xtmp $xt
	    }
	    [set ${data}(xvector)] set $xtmp
	    set ${data}(cxlabel) "d-space"
	} elseif {$mode == "Q"} {
	    set torad [expr 3.14159 / 360.]
	    # wavelength is 2theta for EDS
	    set cnv [expr 6.199 / sin ($torad * [set ${data}(wavelength)])]
	    set pi4 [expr  4. * 3.14159 ]
	    foreach x [[set ${data}(xvector)] range 0 end] {
		set xt 0.0
		catch {set xt [expr $pi4 * $x / $cnv]}
		lappend xtmp $xt
	    }
	    [set ${data}(xvector)] set $xtmp
	    set ${data}(cxlabel) "Q"
	} elseif {$mode == "dspace"} {
	    set torad [expr 3.14159 / 360.]
	    # wavelength is 2theta for EDS
	    set cnv [expr 6.199 / sin ($torad * [set ${data}(wavelength)])]
	    foreach x [[set ${data}(xvector)] range 0 end] {
		set xt 9999
		catch {set xt [expr $cnv / $x]}
		lappend xtmp $xt
	    }
	    [set ${data}(xvector)] set $xtmp
	    set ${data}(cxlabel) "d-space"
	}
	return
    } else {
	tk_dialog .warn "No conversion" \
	"Unable to convert [set ${data}(xunits)] data to \
	$mode units: conversion not programmed" warning 0 OK
	return
    }
}

#-------------------------------------------------------------------------
# file box stuff
#-------------------------------------------------------------------------
# make a box to select the data sets
proc makeselectbox {box} {
    pack [listbox $box.1 -yscrollcommand "$box.scroll set" \
	    -height 8 -width 15 -bd 2 -relief raised ] -side left
    $box.1 config -selectmode extended -exportselection 0
    pack [scrollbar $box.scroll -command "$box.1 yview"] \
	    -side left -fill y
    bind $box.1 <Double-Button-1> "getfilelist $box; displaylist"
    bind $box.1 <Triple-Button-1> \
	    "$box.1 selection set 0 end; getfilelist $box; displaylist"
    bind $box.1 <Button-3> \
	    "$box.1 selection set 0 end; getfilelist $box; displaylist"
    bind $box.1 <Control-Button-3> "deletelist $box; getfilelist $box; displaylist"
}
set helplist(FileSelectionTricks) {
Where a list of datasets is shown, files are selected by 
a left-click. Holding control down and a left-click 
(control-left-click) causes the selected to be selected 
or unselected. Holding the shift down and dragging the 
left mouse button (shift-left-drag) causes a consecutive 
list of files to be selected.

Double-left click causes the current file(s) to be 
selected and plotted (OK to combine with shift or control). 

A triple-left click or a right-click (single) causes all 
files to be selected and plotted.

Control-right-click causes the selected files to be 
deleted from the program (not from disk). You are prompted 
first.}

# update the file list
proc updateselectbox {box {mode extended}} {
    global graph
    $box.1 delete 0 end
    eval  $box.1 insert 0 $graph(datalist)
    $box.1 config -selectmode $mode
}

# update and scroll so the last entry is showing
proc showlastentry {filebox} {
    global command
    updateselectbox $filebox
    $filebox.1 see end 
}
# set graph(plotlist) to the names of the selected entries
proc getfilelist {box} {
    global graph
    set graph(plotlist) {}
    foreach num [$box.1 curselection ] {
	lappend graph(plotlist) [$box.1 get $num]
    }
}
# deletelist deletes selected files
proc deletelist {box} {
    global graph 
    set filelist {}
    foreach entry [$box.1 curselection] {
	lappend filelist [lindex $graph(datalist) $entry]
    }
    if {[tk_dialog .warn "Delete?" \
	    "OK to delete files:\n$filelist" warning 0 OK Cancel] != 0} return
	
    foreach entry [lsort -decreasing -integer [$box.1 curselection]] {
	set file [lindex $graph(datalist) $entry]
	set graph(datalist) [lreplace $graph(datalist) $entry $entry]
    }
    updateselectbox $box
}

# displaylist plots the entries in graph(plotlist)
proc displaylist {} {
    global graph command
    eval $graph(blt) element delete [$graph(blt) element names]
    eval $graph(blt) marker delete  [$graph(blt) marker names]
    set firstpass 1
    foreach data $graph(plotlist) {
	global $data ${data}(xvector) ${data}(yvector)
	if {$firstpass} {
	    $graph(blt) xaxis configure -title [set ${data}(cxlabel)]
	    $graph(blt) yaxis configure -title [set ${data}(cylabel)]
	    set firstpass 0
	}
	# expression to select each color from the list 
	$graph(blt) element create $data \
		-linewidth [set ${data}(line)] \
		-color [set ${data}(color)] -symbol [set ${data}(symbol)] \
		-xdata [set ${data}(xvector)] -ydata [set ${data}(yvector)] \
		-pixels [expr 0.125*[set ${data}(symsize)]]i
#		-activecolor black 
	 [set ${data}(xvector)] notify now
	 [set ${data}(yvector)] notify now
    }
}

#-------------------------------------------------------------------------
# export current plot to Grace
#-------------------------------------------------------------------------
if {$tcl_platform(platform) == "unix"} {
    set graph(gracefile) /tmp/cmpr_grace.agr
} else {
    set graph(gracefile) C:/cmprgrac.agr
}

proc makeGraceExport {} {
    global tcl_platform graph
    catch {toplevel .export}
    raise .export
    eval destroy [grid slaves .export]
    set col 5
    grid [label .export.1a -text Title:] -column 1 -row 1
    grid [entry .export.1b -width 60 -textvariable graph(title)] \
	    -column 2 -row 1 -columnspan 4
    grid [label .export.2a -text Subtitle:] -column 1 -row 2
    grid [entry .export.2b -width 60 -textvariable graph(subtitle)] \
	    -column 2 -row 2 -columnspan 4
    grid [label .export.3a -text "File name:"] -column 1 -row 3
    grid [entry .export.3b -width 60 -textvariable graph(gracefile)] \
	    -column 2 -row 3 -columnspan 4
    grid [button .export.c -text "Close" -command "destroy .export"] \
	    -column [incr col -1] -row 4
    if {$tcl_platform(platform) == "unix"} {
	grid [button .export.d -text "Export & \nstart grace" \
		-command "WriteGrace 1"] \
		-column [incr col -1] -row 4
    }
    grid [button .export.e -text "Export" -command "WriteGrace 0"] \
	    -column [incr col -1] -row 4
}

proc WriteGrace {fork} {
    global graph
    ExportGraceList
    if $fork {
	set err [catch {exec xmgrace $graph(gracefile) &} errmsg]
	if $err {
	    tk_dialog .err "Grace Error" "File $graph(gracefile) created\n\nError running Grace: $errmsg" \
		    error 0 Continue
	}    
    } else {
	tk_dialog .ok " Export Grace" "File $graph(gracefile) created" \
		"" 0 OK
    }
}
proc ExportGraceList {} {
    global graph
    array set gracecolormap {
	black 1 red 2 green 3 blue 4 yellow 5 cyan 9 magenta 10
    }
    array set gracesymbolmap {
	none 0 square 2 circle 1 diamond 3
	plus 8 cross 9 splus 8 scross 9
    }
    array set gracesymbolwidthmap {
	plus 4.0 cross 4.0
    }

    set fp [open $graph(gracefile) w]
    puts $fp {# Grace project file written by CMPR}
    puts $fp {@version 50005}
    puts $fp {#}
    set filenum 0
    foreach data $graph(plotlist) {
	global $data ${data}(xvector) ${data}(yvector)
	set nam s$filenum
	puts $fp "@$nam comment \"[set ${data}(title)]\""
	puts $fp "@$nam legend \"$data\""
	puts $fp "@$nam errorbar off"
	puts $fp "@$nam symbol fill pattern 1"
	# color
	set colnum 12
	catch {set colnum $gracecolormap([set ${data}(color)])}
	puts $fp "@$nam symbol color $colnum"
	puts $fp "@$nam symbol fill color $colnum"
	puts $fp "@$nam line color $colnum"
	if {[set ${data}(type)] == "xy"} {
	    # line type
	    if {[set ${data}(line)] == 0} {
		puts $fp "@$nam line type 0"
	    } else {
		puts $fp "@$nam line type 1"
		puts $fp "@$nam line linewidth [set ${data}(line)]"
	    }
	    # symbol type/width/size
	    set swidth 1.0
	    set sym 10
	    catch {set sym $gracesymbolmap([set ${data}(symbol)])}
	    catch {set swidth $gracesymbolwidthmap([set ${data}(symbol)])}
	    puts $fp "@$nam symbol $sym"
	    puts $fp "@$nam symbol linewidth $swidth"
	    puts $fp "@$nam symbol size [set ${data}(symsize)]"
	    puts $fp "@type xydy"
	    foreach x [ [set ${data}(xvector)] range 0 end] \
		    y [ [set ${data}(yvector)] range 0 end] \
		    e [ [set ${data}(esdvec)] range 0 end] {
		if {$e == ""} {set e 0}
		puts $fp "$x $y $e"
	    }
	    puts $fp "&"
	} elseif {[set ${data}(type)] == "peaks"} {
	    # line type
	    puts $fp "@$nam line type 0"
 	    puts $fp "@$nam baseline type 0"
	    if {[set ${data}(line)] == 0} {
		puts $fp "@$nam symbol pattern 0"
		puts $fp "@$nam symbol fill pattern 0"
	    } else {
		puts $fp "@$nam symbol pattern 1"
		puts $fp "@$nam symbol fill pattern 1"
		puts $fp "@$nam symbol size 0.[set ${data}(line)]"
	    
	    }
	    # symbols are ignored
	    puts $fp "@type bar"
	    set i 0
	    foreach x [ [set ${data}(xvector)] range 0 end] \
		    y [ [set ${data}(yvector)] range 0 end] {
		incr i
		if {$i %3 == 2} {puts $fp "$x $y"}
	    }
	    puts $fp "&"
	}
	if {$filenum == 0} {
	    # save axes labels from first file
	    set xlbl [set ${data}(cxlabel)]
	    set ylbl [set ${data}(cylabel)]
	}
	incr filenum
    }
    # now do scaling
    puts $fp {@with g0}
    puts $fp {@autoscale}
    puts $fp {# is this needed?}
    puts $fp {@    stack world 0, 0, 0, 0}
    puts $fp {@    view xmin 0.150000}
    puts $fp {@    view xmax 1.150000}
    puts $fp {@    view ymin 0.150000}
    puts $fp {@    view ymax 0.850000}
    foreach opt  {-min  -max   -min  -max} \
	    axis {xaxis xaxis yaxis yaxis} \
	    var  { xmin  xmax  ymin  ymax} {
	set val [$graph(blt) $axis cget $opt]
	if {$val != ""} {puts $fp "@world $var $val"}
    }
    puts $fp {@    legend loctype view}
    puts $fp {@    legend x1 0.9}
    puts $fp {@    legend y1 0.8}
    puts $fp "@title \"$graph(title)\""
    puts $fp "@subtitle \"$graph(subtitle)\""
    regsub "2theta" $xlbl {2\f{Symbol}q} xlbl
    regsub "2 Theta" $xlbl  {2\f{Symbol}q} xlbl
    if {$xlbl == "Q"} {set xlbl { Q, \305}}
    puts $fp "@xaxis label \"$xlbl\""
    puts $fp "@yaxis label \"$ylbl\""
    if {$graph(legend)} {
	puts $fp "@legend on"
    } else {
	puts $fp "@legend off"
    }
    puts $fp "@g0 hidden false"
    puts $fp "@autoticks"
    close $fp
}

#-------------------------------------------------------------------------
# rework later
#-------------------------------------------------------------------------

proc definecolors {} {
    global colorlist linelist ncolors nlines colorindex lineindex
    if {[regexp .*color [winfo visual .]]} {
	set colorlist "red green blue magenta cyan"
	set linelist "none"
    } { 
	#set linelist "square circle diamond plus cross splus scross"
	set linelist "diamond splus scross"
	set colorlist "black"
    }
    set ncolors [llength $colorlist]
    set nlines [llength $linelist]
    set colorindex -1
    set lineindex -1
}
definecolors
